@@ -24,6 +24,8 @@ except ImportError:
     # Allow parts of ecl2df to work without OPM:
     pass
 
+from ecl2df import getLogger_ecl2csv
+
 from .common import (
     get_wells_matching_template,
     merge_zones,
@@ -101,7 +103,7 @@ def deck2dfs(
     welspecs = {}
     date = start_date  # DATE column will always be there, but can contain NaN/None
     for idx, kword in enumerate(deck):
-        if kword.name == "DATES" or kword.name == "START":
+        if kword.name in ["DATES", "START"]:
             for rec in kword:
                 date = parse_opmio_date_rec(rec)
                 logger.info("Parsing at date %s", str(date))
@@ -183,8 +185,7 @@ def deck2dfs(
                 rec_data = parse_opmio_deckrecord(rec, "WELOPEN")
                 rec_data["DATE"] = date
                 rec_data["KEYWORD_IDX"] = idx
-                if rec_data["STATUS"] not in ["OPEN", "SHUT", "STOP", "AUTO"]:
-                    rec_data["STATUS"] = "SHUT"
+                if rec_data["STATUS"] not in ["OPEN", "SHUT", "STOP", "AUTO", "POPN"]:
                     logger.warning(
                         (
                             "WELOPEN status %s is not a valid "
@@ -192,6 +193,7 @@ def deck2dfs(
                         ),
                         rec_data["STATUS"],
                     )
+                    rec_data["STATUS"] = "SHUT"
                 welopenrecords.append(rec_data)
         elif kword.name == "WELSEGS":
             # First record contains meta-information for well
@@ -279,6 +281,7 @@ def deck2dfs(
         COMPDAT=compdat_df,
         COMPSEGS=compsegs_df,
         WELSEGS=welsegs_df,
+        WELOPEN=welopen_df,
         WLIST=wlist_df,
         WSEGSICD=wsegsicd_df,
         WSEGAICD=wsegaicd_df,
@@ -286,47 +289,6 @@ def deck2dfs(
     )
 
 
-def postprocess():
-    """Postprocessing of the compdat data, merging.
-
-    This function is NOT FINISHED"""
-    # compdat_df = pd.read_csv("compdat.csv")
-    compsegs_df = pd.read_csv("compsegs.csv")
-    welsegs_df = pd.read_csv("welsegs.csv")
-
-    #  We need different handling of ICD's and non-ICD wells due
-    #  to the complex WELSEGS structure:
-    #
-    # ICD wells:
-    # 1. First compdata is merged with compsegs (non-ICD
-    #    should be stripped away).
-    # 2. Then that product is merged with welsegs on 'branch'
-    # 3. Then that product is merged again with welsegs, where
-    #    we join on 'join_segment' and 'segment'
-    # 4. Then we finally have the mapping between completed
-    #    cells and branch number
-    #
-    # Non-ICD wells:
-    # 1. Merge compdata and compsegs
-    # 2. Then we are ready.. compsegs contains the correct branch number
-
-    # compdatsegs = pd.merge(compdat_df,
-    #                        compsegs_df, on=["date", "well", "i", "j", "k"])
-    # WARNING: Only correct for dual-branch wells,
-    # not triple-branach wells with ICD..
-    compsegs_icd_df = compsegs_df[compsegs_df.branch > 2]
-    # icd_wells = compsegs_icd_df.well.unique()
-    compdatsegwel_icd_df = pd.merge(
-        compsegs_icd_df, welsegs_df, on=["date", "well", "branch"]
-    )
-    del compdatsegwel_icd_df["segment"]  # we don't need this
-    compdatsegwel_icd_df.rename(columns={"branch": "icd_branch"}, inplace=True)
-    compdatsegwel_icd_df.rename(columns={"join_segment": "segment"}, inplace=True)
-    # alldata_icd = pd.merge(
-    #     compdatsegwel_icd_df, welsegs_df, on=["date", "well", "segment"]
-    # )
-
-
 def expand_welopen_wildcards(welopen_df: pd.DataFrame, compdat_df: pd.DataFrame):
     """Expand rows in welopen with well names containing wildcard characters,
     with the correct wells from compdat_df that was defined at that date
@@ -459,19 +421,19 @@ def unroll_complump(complump_df: pd.DataFrame) -> pd.DataFrame:
         return complump_df
 
     for _, row in complump_df.iterrows():
-        val_I = int(row["I"])
-        val_J = int(row["J"])
-        val_K1 = int(row["K1"])
-        val_K2 = int(row["K2"])
-        if val_I < 0 or val_J < 0 or val_K1 < 0 or val_K2 < 0:
+        val_i = int(row["I"])
+        val_j = int(row["J"])
+        val_k1 = int(row["K1"])
+        val_k2 = int(row["K2"])
+        if val_i < 0 or val_j < 0 or val_k1 < 0 or val_k2 < 0:
             raise ValueError(
                 f"Negative values for COMPLUMP coordinates are not allowed: {row}"
             )
-        elif val_I == 0 or val_J == 0 or val_K1 == 0 or val_K2 == 0:
+        if val_i == 0 or val_j == 0 or val_k1 == 0 or val_k2 == 0:
             raise ValueError(
                 f"Defaulted COMPLUMP coordinates are not supported in ecl2df: {row}"
             )
-        elif val_K2 < val_K1:
+        if val_k2 < val_k1:
             raise ValueError(f"K2 must be equal to or greater than K1: {row}")
     return unrolldf(complump_df)
 
@@ -528,7 +490,7 @@ def expand_wlist(wlist_df: pd.DataFrame) -> pd.DataFrame:
     currentdate = wlist_df["DATE"].min()
     new_records = []
 
-    for idx, wlist_record in wlist_df.iterrows():
+    for _, wlist_record in wlist_df.iterrows():
         date = wlist_record["DATE"]
         if date > currentdate:
             # Store current state
@@ -602,7 +564,9 @@ def expand_wlist(wlist_df: pd.DataFrame) -> pd.DataFrame:
                     )
                 )
             )
-            for wlist in currentstate.keys():
+            for (
+                wlist
+            ) in currentstate.keys():  # pylint: disable=consider-iterating-dictionary
                 if wlist == wlist_record["NAME"]:
                     continue
                 currentstate[wlist] = " ".join(
@@ -659,10 +623,10 @@ def expand_complump_in_welopen_df(
     """
 
     if (
-        welopen_df.empty
-        or welopen_df is None
-        or complump_df.empty
+        welopen_df is None
+        or welopen_df.empty
         or complump_df is None
+        or complump_df.empty
     ):
         return welopen_df
 
@@ -678,20 +642,34 @@ def expand_complump_in_welopen_df(
         else:
             # Found a row that refers to complump numbers
             # Check that the cumplump numbers are ok:
-            C1, C2 = int(row["C1"]), int(row["C2"])
-            if C1 < 0 or C2 < 0:
+            c_1, c_2 = int(row["C1"]), int(row["C2"])
+            if c_1 < 0 or c_2 < 0:
                 raise ValueError(f"Negative values for C1/C2 is not allowed: {row}")
-            if C1 == 0 or C2 == 0:
-                raise ValueError(f"Defaults (zero) for C1/C2 is not implemented: {row}")
-            if C2 < C1:
+            if c_1 == 0 or c_2 == 0:
+                raise ValueError(f"Zeros for C1/C2 is not implemented: {row}")
+            if c_2 < c_1:
                 raise ValueError(f"C2 must be equal or greater than C1: {row}")
 
             relevant_complump_df = complump_df[
                 (complump_df["DATE"] <= row["DATE"])
                 & (complump_df["WELL"] == row["WELL"])
-                & (complump_df["N"] >= C1)
-                & (complump_df["N"] <= C2)
+                & (complump_df["N"] >= c_1)
+                & (complump_df["N"] <= c_2)
             ]
+            # If I, J and K is also present in the WELOPEN record
+            # we should "and" the complump filter and the i,j,k filter:
+            if (
+                {"I", "J", "K"}.issubset(row.keys())
+                and row["I"]
+                and row["J"]
+                and row["K"]
+            ):
+                relevant_complump_df = relevant_complump_df[
+                    (relevant_complump_df["I"] == row["I"])
+                    & (relevant_complump_df["J"] == row["J"])
+                    & (relevant_complump_df["K1"] == row["K"])
+                    & (relevant_complump_df["K2"] == row["K"])
+                ]
             for _, complump_row in relevant_complump_df.iterrows():
                 if complump_row["K1"] != complump_row["K2"]:
                     raise ValueError(
@@ -767,9 +745,15 @@ def applywelopen(
       /
 
     This deck would define two wells where OP1 and OP2 have two connected grid cells
-    each. Although the COMPDAT defines all connections to be open, WELOPEN overwrites
-    this: all connections in OP1 will be SHUT and in OP2 the upper connection will
-    be SHUT.
+    each. The first welopen statment acts on the whole well, closing both the well and
+    the connections. If this statement used STOP instead of SHUT, the connections would
+    be left open. The second welopen statement acts on a single connection. Here SHUT
+    and STOP would give the same result. This behavior has been proven to be correct
+    in the simulator. The Eclipse manual states that 'If items 3 - 7 are all defaulted,
+    the Open/Shut/Stop command applies to the well, leaving the connections unchanged',
+    but this has been proven to be wrong. The state of the connection can be tested
+    by looking at the CPI summary vectors. The connection is SHUT if CPI==0 and OPEN
+    if CPI>0.
 
     WELOPEN can also be used at different dates and changes therefore the state of
     connections without explicit use of the COMPDAT keyword. This function translates
@@ -799,14 +783,28 @@ def applywelopen(
     welopen_df = expand_complump_in_welopen_df(welopen_df, complump_df)
 
     for _, row in welopen_df.iterrows():
+        acts_on_well = False
         if (row["I"] is None and row["J"] is None and row["K"] is None) or (
             row["I"] <= 0 and row["J"] <= 0 and row["K"] <= 0
         ):
+            # Applies to all connections when the completion range
+            # is set zero or negative.
             previous_state = compdat_df[
                 (compdat_df["WELL"] == row["WELL"])
                 & (compdat_df["KEYWORD_IDX"] < row["KEYWORD_IDX"])
             ].drop_duplicates(subset=["I", "J", "K1", "K2"], keep="last")
-        elif row["I"] and row["J"] and row["K"]:
+            acts_on_well = True
+        elif (
+            row["I"]
+            and row["J"]
+            and row["K"]
+            and row["C1"] is None
+            and row["C2"] is None
+        ):
+            # The compdat dataframe is assumed unrolled
+            # so that K1 is always equal to K2. Any openings of lumped
+            # connections (C1 and C2) should already be translated to
+            # I, J, and K when we get here.
             previous_state = compdat_df[
                 (compdat_df["WELL"] == row["WELL"])
                 & (compdat_df["KEYWORD_IDX"] < row["KEYWORD_IDX"])
@@ -815,11 +813,6 @@ def applywelopen(
                 & (compdat_df["K1"] == row["K"])
                 & (compdat_df["K2"] == row["K"])
             ].drop_duplicates(subset=["I", "J", "K1", "K2"], keep="last")
-        elif row["I"] <= 0 and row["J"] <= 0 and row["K"] <= 0:
-            previous_state = compdat_df[
-                (compdat_df["WELL"] == row["WELL"])
-                & (compdat_df["KEYWORD_IDX"] < row["KEYWORD_IDX"])
-            ].drop_duplicates(subset=["I", "J", "K1", "K2"], keep="last")
         else:
             raise ValueError(
                 "A WELOPEN keyword contains data that could not be parsed. "
@@ -840,7 +833,16 @@ def applywelopen(
         # underlying problem is that the opm-common definitions for the state of a
         # well in COMPDAT and WELOPEN are not identical. These translation steps can
         # be dropped when unity in the opm-common keyword definitions is reached.
-        new_state["OP/SH"] = row["STATUS"]
+        new_state["OP/SH"] = row["STATUS"].replace("POPN", "OPEN")
+
+        # If the welopen statement acts on the whole well, then STOP closes the well
+        # but opens the connections. If the welopen statement applies to selected
+        # connections, then STOP means the same as SHUT.
+        if acts_on_well:
+            new_state["OP/SH"] = new_state["OP/SH"].replace("STOP", "OPEN")
+        else:
+            new_state["OP/SH"] = new_state["OP/SH"].replace("STOP", "SHUT")
+
         new_state["KEYWORD_IDX"] = row["KEYWORD_IDX"]
         new_state["DATE"] = row["DATE"]
 
@@ -882,8 +884,9 @@ def fill_parser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
 
 def compdat_main(args):
     """Entry-point for module, for command line utility"""
-    if args.verbose:
-        logging.basicConfig(level=logging.INFO)
+    logger = getLogger_ecl2csv(  # pylint: disable=redefined-outer-name
+        __name__, vars(args)
+    )
     eclfiles = EclFiles(args.DATAFILE)
     compdat_df = df(eclfiles, initvectors=args.initvectors)
     write_dframe_stdout_file(compdat_df, args.output, index=False, caller_logger=logger)
