@@ -1,23 +1,24 @@
 """Test module for ecl2df.grid"""
-
 import datetime
-import sys
+import os
 from pathlib import Path
 
 import numpy as np
 import pandas as pd
+import pyarrow
 import pytest
 
 from ecl2df import common, ecl2csv, grid
 from ecl2df.eclfiles import EclFiles
 
 TESTDIR = Path(__file__).absolute().parent
-DATAFILE = str(TESTDIR / "data/reek/eclipse/model/2_R001_REEK-0.DATA")
+REEK = str(TESTDIR / "data/reek/eclipse/model/2_R001_REEK-0.DATA")
+EIGHTCELLS = str(TESTDIR / "data/eightcells/EIGHTCELLS")
 
 
 def test_gridgeometry2df(mocker):
     """Test that dataframes are produced"""
-    eclfiles = EclFiles(DATAFILE)
+    eclfiles = EclFiles(REEK)
     grid_geom = grid.gridgeometry2df(eclfiles)
 
     assert isinstance(grid_geom, pd.DataFrame)
@@ -66,7 +67,7 @@ def test_wrongfile():
 def test_gridzonemap():
     """Check that zonemap can be merged automatically be default, and also
     that there is some API for supplying the zonemap directly as a dictionary"""
-    eclfiles = EclFiles(DATAFILE)
+    eclfiles = EclFiles(EIGHTCELLS)
     grid_geom = grid.gridgeometry2df(eclfiles, zonemap=None)
 
     default_zonemap = grid_geom["ZONE"]
@@ -104,7 +105,7 @@ def test_gridzonemap():
 
 
 def test_merge_initvectors():
-    eclfiles = EclFiles(DATAFILE)
+    eclfiles = EclFiles(REEK)
     assert grid.merge_initvectors(eclfiles, pd.DataFrame(), []).empty
     foo_df = pd.DataFrame([{"FOO": 1}])
     pd.testing.assert_frame_equal(grid.merge_initvectors(eclfiles, foo_df, []), foo_df)
@@ -135,7 +136,7 @@ def test_merge_initvectors():
 
 def test_init2df():
     """Test that dataframe with INIT vectors can be produced"""
-    eclfiles = EclFiles(DATAFILE)
+    eclfiles = EclFiles(REEK)
     init_df = grid.init2df(eclfiles)
 
     assert isinstance(init_df, pd.DataFrame)
@@ -144,7 +145,7 @@ def test_init2df():
     assert "PORO" in init_df
     assert "PORV" in init_df
 
-    # The KRO data from the INIT file contains only NaN's,
+    # The KRO data from the INIT file in Reek contains only NaN's,
     # but libecl gives out a large negative integer/float.
     # ecl2df should ensure this comes out as a NaN (but it
     # should be allowed later to drop columns which have only NaNs))
@@ -154,7 +155,7 @@ def test_init2df():
 
 def test_grid_df():
     """Test that dataframe with INIT vectors and coordinates can be produced"""
-    eclfiles = EclFiles(DATAFILE)
+    eclfiles = EclFiles(EIGHTCELLS)
     grid_df = grid.df(eclfiles)
 
     assert isinstance(grid_df, pd.DataFrame)
@@ -178,9 +179,9 @@ def test_grid_df():
     )
 
 
-def test_df2ecl(tmpdir):
+def test_df2ecl(tmp_path):
     """Test if we are able to output include files for grid data"""
-    eclfiles = EclFiles(DATAFILE)
+    eclfiles = EclFiles(REEK)
     grid_df = grid.df(eclfiles)
 
     fipnum_str = grid.df2ecl(grid_df, "FIPNUM", dtype=int)
@@ -216,7 +217,7 @@ def test_df2ecl(tmpdir):
     assert "3333" in fipnum_big_str
     assert len(fipnum_big_str) > len(fipnum_str)
 
-    tmpdir.chdir()
+    os.chdir(tmp_path)
     grid.df2ecl(grid_df, ["PERMX", "PERMY", "PERMZ"], dtype=float, filename="perm.inc")
     assert Path("perm.inc").is_file()
     incstring = open("perm.inc").readlines()
@@ -263,7 +264,7 @@ def test_df2ecl_mock():
 
 def test_subvectors():
     """Test that we can ask for a few vectors only"""
-    eclfiles = EclFiles(DATAFILE)
+    eclfiles = EclFiles(EIGHTCELLS)
     init_df = grid.init2df(eclfiles, "PORO")
     assert "PORO" in init_df
     assert "PERMX" not in init_df
@@ -309,7 +310,7 @@ def test_dropconstants():
 
 def test_df():
     """Test the df function"""
-    eclfiles = EclFiles(DATAFILE)
+    eclfiles = EclFiles(REEK)
     # assert error..
     with pytest.raises(TypeError):
         # pylint: disable=no-value-for-parameter
@@ -411,20 +412,23 @@ def test_df():
     # but not for now.
 
 
-def test_main(tmpdir):
+def test_main(tmp_path, mocker):
     """Test command line interface"""
-    tmpcsvfile = tmpdir / "eclgrid.csv"
-    sys.argv = [
-        "ecl2csv",
-        "grid",
-        DATAFILE,
-        "-o",
-        str(tmpcsvfile),
-        "--rstdates",
-        "first",
-        "--vectors",
-        "PORO",
-    ]
+    tmpcsvfile = tmp_path / "eclgrid.csv"
+    mocker.patch(
+        "sys.argv",
+        [
+            "ecl2csv",
+            "grid",
+            EIGHTCELLS,
+            "-o",
+            str(tmpcsvfile),
+            "--rstdates",
+            "first",
+            "--vectors",
+            "PORO",
+        ],
+    )
     ecl2csv.main()
     assert Path(tmpcsvfile).is_file()
     disk_df = pd.read_csv(str(tmpcsvfile))
@@ -432,19 +436,22 @@ def test_main(tmpdir):
     Path(tmpcsvfile).unlink()
 
     # Do again with also restarts and multiple vectors:
-    sys.argv = [
-        "ecl2csv",
-        "grid",
-        "--verbose",
-        DATAFILE,
-        "-o",
-        str(tmpcsvfile),
-        "--rstdates",
-        "2001-02-01",
-        "--vectors",
-        "PORO",
-        "PERMX",
-    ]
+    mocker.patch(
+        "sys.argv",
+        [
+            "ecl2csv",
+            "grid",
+            "--verbose",
+            EIGHTCELLS,
+            "-o",
+            str(tmpcsvfile),
+            "--rstdates",
+            "2000-01-01",
+            "--vectors",
+            "PORO",
+            "PERMX",
+        ],
+    )
     ecl2csv.main()
     assert Path(tmpcsvfile).is_file()
     disk_df = pd.read_csv(str(tmpcsvfile))
@@ -454,7 +461,9 @@ def test_main(tmpdir):
     Path(tmpcsvfile).unlink()
 
     # Test with constants dropping
-    sys.argv = ["ecl2csv", "grid", DATAFILE, "-o", str(tmpcsvfile), "--dropconstants"]
+    mocker.patch(
+        "sys.argv", ["ecl2csv", "grid", REEK, "-o", str(tmpcsvfile), "--dropconstants"]
+    )
     ecl2csv.main()
     assert Path(tmpcsvfile).is_file()
     disk_df = pd.read_csv(str(tmpcsvfile))
@@ -464,9 +473,30 @@ def test_main(tmpdir):
     assert not disk_df.empty
 
 
+def test_main_arrow(tmp_path, mocker):
+    """Check that we can export grid in arrow format"""
+    mocker.patch(
+        "sys.argv",
+        ["ecl2csv", "grid", "--arrow", EIGHTCELLS, "-o", str(tmp_path / "grid.arrow")],
+    )
+    ecl2csv.main()
+
+    # Obtain the CSV version for comparison:
+    mocker.patch(
+        "sys.argv", ["ecl2csv", "grid", EIGHTCELLS, "-o", str(tmp_path / "grid.csv")]
+    )
+    ecl2csv.main()
+
+    # Read from disk and verify similarity
+    disk_frame_arrow = pyarrow.feather.read_table(tmp_path / "grid.arrow").to_pandas()
+    disk_frame_csv = pd.read_csv(tmp_path / "grid.csv")
+
+    pd.testing.assert_frame_equal(disk_frame_arrow, disk_frame_csv, check_dtype=False)
+
+
 def test_get_available_rst_dates():
     """Test the support of dates in restart files"""
-    eclfiles = EclFiles(DATAFILE)
+    eclfiles = EclFiles(REEK)
     # rstfile = eclfiles.get_rstfile()
 
     alldates = grid.get_available_rst_dates(eclfiles)
@@ -538,7 +568,7 @@ def test_get_available_rst_dates():
 
 def test_rst2df():
     """Test producing dataframes from restart files"""
-    eclfiles = EclFiles(DATAFILE)
+    eclfiles = EclFiles(REEK)
     assert grid.rst2df(eclfiles, "first").shape == (35817, 24)
     assert grid.rst2df(eclfiles, "last").shape == (35817, 24)
     assert grid.rst2df(eclfiles, "all").shape == (35817, 23 * 4 + 1)
