@@ -1,7 +1,7 @@
 """Test module for pvt"""
 
 import io
-import logging
+import os
 import subprocess
 from pathlib import Path
 
@@ -11,11 +11,17 @@ import pytest
 from ecl2df import csv2ecl, ecl2csv, pvt
 from ecl2df.eclfiles import EclFiles
 
-TESTDIR = Path(__file__).absolute().parent
-DATAFILE = str(TESTDIR / "data/reek/eclipse/model/2_R001_REEK-0.DATA")
+try:
+    import opm  # noqa
+except ImportError:
+    pytest.skip(
+        "OPM is not installed, nothing relevant in here then",
+        allow_module_level=True,
+    )
 
-logger = logging.getLogger("")
-logger.setLevel(logging.DEBUG)
+TESTDIR = Path(__file__).absolute().parent
+REEK = str(TESTDIR / "data/reek/eclipse/model/2_R001_REEK-0.DATA")
+EIGHTCELLS = str(TESTDIR / "data/eightcells/EIGHTCELLS.DATA")
 
 
 def test_pvto_strings():
@@ -153,7 +159,7 @@ PVDO
 def test_pvt_reek():
     """Test that the Reek PVT input can be parsed individually"""
 
-    eclfiles = EclFiles(DATAFILE)
+    eclfiles = EclFiles(REEK)
     pvto_df = pvt.pvto_fromdeck(eclfiles.get_ecldeck())
     assert "PVTNUM" in pvto_df
     assert "PRESSURE" in pvto_df
@@ -285,7 +291,7 @@ PVTG
 
 def test_density():
     """Test that DENSITY can be parsed from files and from strings"""
-    eclfiles = EclFiles(DATAFILE)
+    eclfiles = EclFiles(REEK)
     density_df = pvt.density_fromdeck(eclfiles.get_ecldeck())
     assert len(density_df) == 1
     assert "PVTNUM" in density_df
@@ -370,7 +376,7 @@ def test_rock():
 
 def test_df():
     """Test that aggregate dataframes are produced"""
-    eclfiles = EclFiles(DATAFILE)  # Reek dataset
+    eclfiles = EclFiles(REEK)
     pvtdf = pvt.df(eclfiles)
 
     assert not pvtdf.empty
@@ -383,11 +389,13 @@ def test_df():
     assert len(pvtdf["PVTNUM"].unique()) == 1
 
 
-def test_main(tmpdir, mocker):
+def test_main(tmp_path, mocker):
     """Test command line interface"""
-    tmpdir.chdir()
-    tmpcsvfile = str(tmpdir.join("pvt.csv"))
-    mocker.patch("sys.argv", ["ecl2csv", "pvt", "-v", DATAFILE, "-o", tmpcsvfile])
+    os.chdir(tmp_path)
+    tmpcsvfile = tmp_path / "pvt.csv"
+    mocker.patch(
+        "sys.argv", ["ecl2csv", "pvt", "-v", EIGHTCELLS, "-o", str(tmpcsvfile)]
+    )
     ecl2csv.main()
 
     assert Path(tmpcsvfile).is_file()
@@ -397,8 +405,10 @@ def test_main(tmpdir, mocker):
     assert not disk_df.empty
 
     # Write back to include file:
-    incfile = str(tmpdir.join("pvt.inc"))
-    mocker.patch("sys.argv", ["csv2ecl", "pvt", "-v", str(tmpcsvfile), "-o", incfile])
+    incfile = tmp_path / "pvt.inc"
+    mocker.patch(
+        "sys.argv", ["csv2ecl", "pvt", "-v", str(tmpcsvfile), "-o", str(incfile)]
+    )
     csv2ecl.main()
 
     # Reparse the include file on disk back to dataframe
@@ -408,7 +418,7 @@ def test_main(tmpdir, mocker):
     pd.testing.assert_frame_equal(disk_df, disk_inc_df)
 
     # Test entry point towards include file:
-    (Path(tmpdir) / "pvto.inc").write_text(
+    (Path(tmp_path) / "pvto.inc").write_text(
         """PVTO
     0      1 1.0001 1
          200 1.000  1.001 /
@@ -421,7 +431,7 @@ def test_main(tmpdir, mocker):
     assert Path("pvto.csv").is_file()
 
     # Empty data:
-    (Path(tmpdir) / "empty.inc").write_text(
+    (Path(tmp_path) / "empty.inc").write_text(
         """SWOF
     0      1 1.0001 1 /
     /
@@ -432,18 +442,18 @@ def test_main(tmpdir, mocker):
     assert not Path("empty.csv").read_text().strip()
 
 
-def test_magic_stdout(tmpdir):
+def test_magic_stdout(tmp_path):
     """Test writing dataframes and include files to stdout"""
-    tmpdir.chdir()
+    os.chdir(tmp_path)
     result = subprocess.run(
-        ["ecl2csv", "pvt", "-o", "-", DATAFILE], check=True, stdout=subprocess.PIPE
+        ["ecl2csv", "pvt", "-o", "-", EIGHTCELLS], check=True, stdout=subprocess.PIPE
     )
     df_stdout = pd.read_csv(io.StringIO(result.stdout.decode()))
     assert not df_stdout.empty
 
     # Verbose options should not ruin it:
     result = subprocess.run(
-        ["ecl2csv", "pvt", "--verbose", "-o", "-", DATAFILE],
+        ["ecl2csv", "pvt", "--verbose", "-o", "-", EIGHTCELLS],
         check=True,
         stdout=subprocess.PIPE,
     )
@@ -515,9 +525,9 @@ def test_df2ecl_pvto():
     # least it is well defined how to treat it)
 
 
-def test_df2ecl_rock(tmpdir):
+def test_df2ecl_rock(tmp_path):
     """Test generation of ROCK include files from dataframes"""
-    tmpdir.chdir()
+    os.chdir(tmp_path)
 
     rock_df = pd.DataFrame(
         columns=["PVTNUM", "KEYWORD", "PRESSURE", "COMPRESSIBILITY"],
@@ -651,8 +661,12 @@ def test_df2ecl_pvtg():
         pvt.df2ecl_pvtg(pvtg_df.drop("VOLUMEFACTOR", axis="columns"))
 
 
-def test_df2ecl_pvdo():
-    pvdo_df = pd.DataFrame(
+def test_df2ecl_pvdo_pvdg():
+    """Test construction of PVDO and PVDG statements from dataframe.
+
+    The keyword data and code is similar enough to warrant one test
+    for both functions, with the same dataset."""
+    pvdog_df = pd.DataFrame(
         columns=["PRESSURE", "VOLUMEFACTOR", "VISCOSITY", "PVTNUM"],
         data=[
             [400.0, 6, 0.01, 1],
@@ -662,20 +676,33 @@ def test_df2ecl_pvdo():
         ],
     )
 
-    assert "PVDO" in pvt.df2ecl_pvdo(pvdo_df)
-    assert "PVDO" in pvt.df2ecl_pvdo(pvdo_df.assign(KEYWORD="PVDO"))
+    assert "PVDO" in pvt.df2ecl_pvdo(pvdog_df)
+    assert "PVDG" in pvt.df2ecl_pvdg(pvdog_df)
+
+    assert "PVDO" in pvt.df2ecl_pvdo(pvdog_df.assign(KEYWORD="PVDO"))
+    assert "PVDG" in pvt.df2ecl_pvdg(pvdog_df.assign(KEYWORD="PVDG"))
+
     pd.testing.assert_frame_equal(
-        pvt.df(pvt.df2ecl_pvdo(pvdo_df)).drop("KEYWORD", axis="columns"), pvdo_df
+        pvt.df(pvt.df2ecl_pvdo(pvdog_df)).drop("KEYWORD", axis="columns"), pvdog_df
+    )
+    pd.testing.assert_frame_equal(
+        pvt.df(pvt.df2ecl_pvdg(pvdog_df)).drop("KEYWORD", axis="columns"), pvdog_df
     )
 
     # If PVTNUM is missing, the code gives up:
     assert "PVDO" not in pvt.df2ecl_pvdo(
-        pd.concat([pvdo_df, pvdo_df]).drop("PVTNUM", axis="columns")
+        pd.concat([pvdog_df, pvdog_df]).drop("PVTNUM", axis="columns")
+    )
+    assert "PVDG" not in pvt.df2ecl_pvdg(
+        pd.concat([pvdog_df, pvdog_df]).drop("PVTNUM", axis="columns")
     )
 
     # Unless there is only one row:
-    assert "PVDO" in pvt.df2ecl_pvdo(pvdo_df.head(1).drop("PVTNUM", axis="columns"))
+    assert "PVDO" in pvt.df2ecl_pvdo(pvdog_df.head(1).drop("PVTNUM", axis="columns"))
+    assert "PVDG" in pvt.df2ecl_pvdg(pvdog_df.head(1).drop("PVTNUM", axis="columns"))
 
     # Missing column:
     with pytest.raises(KeyError, match="VOLUMEFACTOR"):
-        pvt.df2ecl_pvdo(pvdo_df.drop("VOLUMEFACTOR", axis="columns"))
+        pvt.df2ecl_pvdo(pvdog_df.drop("VOLUMEFACTOR", axis="columns"))
+    with pytest.raises(KeyError, match="VOLUMEFACTOR"):
+        pvt.df2ecl_pvdg(pvdog_df.drop("VOLUMEFACTOR", axis="columns"))
