@@ -5,9 +5,17 @@ import pytest
 
 from ecl2df import EclFiles, compdat
 
+try:
+    import opm  # noqa
+except ImportError:
+    pytest.skip(
+        "OPM is not installed",
+        allow_module_level=True,
+    )
+
 WELOPEN_CASES = [
-    # Simplest possible WELOPEN case
-    (
+    # WELOPEN SHUT closes both well and connections
+    pytest.param(
         """
     DATES
      1 JAN 2000 /
@@ -25,9 +33,10 @@ WELOPEN_CASES = [
                 [datetime.date(2000, 1, 1), "OP1", 1, 1, 1, 1, "SHUT"],
             ],
         ),
+        id="welopen-shut",
     ),
-    # Test the defaults handling in WELOPEN:
-    (
+    # WELOPEN SHUT with explicit defaults, still closes both well and connections
+    pytest.param(
         """
     DATES
      1 JAN 2000 /
@@ -45,9 +54,10 @@ WELOPEN_CASES = [
                 [datetime.date(2000, 1, 1), "OP1", 1, 1, 1, 1, "SHUT"],
             ],
         ),
+        id="welopen-shut-explicit-defaults",
     ),
-    # Test the defaults handling in WELOPEN (zero value means apply to all)
-    (
+    # Test the zero handling in WELOPEN (zero value means apply to all)
+    pytest.param(
         """
     DATES
      1 JAN 2000 /
@@ -65,9 +75,10 @@ WELOPEN_CASES = [
                 [datetime.date(2000, 1, 1), "OP1", 1, 1, 1, 1, "SHUT"],
             ],
         ),
+        id="zero-values",
     ),
     # Test the defaults handling in WELOPEN (default values are negative)
-    (
+    pytest.param(
         """
     DATES
      1 JAN 2000 /
@@ -85,9 +96,10 @@ WELOPEN_CASES = [
                 [datetime.date(2000, 1, 1), "OP1", 1, 1, 1, 1, "SHUT"],
             ],
         ),
+        id="negative-values",
     ),
-    # Fail with ValueError when both I,J,K (3-5) and completions number (6-7)
-    # are defined in WELOPEN
+    # When item 3-7 are defaulted, the action applies to the well and not the
+    # connections. Thus they should be kept open.
     pytest.param(
         """
     DATES
@@ -97,8 +109,80 @@ WELOPEN_CASES = [
      'OP1' 1 1 1 1 'OPEN' /
     /
     WELOPEN
-     -- This also specifies lumped connections, which will give crash
-     'OP1' 'SHUT' 1 1 1 1 1 /
+     'OP1' 'STOP' /
+    /
+    """,
+        pd.DataFrame(
+            columns=["DATE", "WELL", "I", "J", "K1", "K2", "OP/SH"],
+            data=[
+                [datetime.date(2000, 1, 1), "OP1", 1, 1, 1, 1, "OPEN"],
+            ],
+        ),
+        id="welopen-stop-on-well",
+    ),
+    pytest.param(
+        """
+    DATES
+     1 JAN 2000 /
+    /
+    COMPDAT
+     'OP1' 1 1 1 1 'OPEN' /
+    /
+    WELOPEN
+     'OP1' 'STOP' 1* 1* 1* /
+    /
+    """,
+        pd.DataFrame(
+            columns=["DATE", "WELL", "I", "J", "K1", "K2", "OP/SH"],
+            data=[
+                [datetime.date(2000, 1, 1), "OP1", 1, 1, 1, 1, "OPEN"],
+            ],
+        ),
+        id="welopen-stop-on-well-explicit-defaults",
+    ),
+    # In this test, the well connection is first SHUT, but then
+    # actually opened by applying STOP on the well. The well is still
+    # closed but the connection is open. Note that applying STOP to
+    # the connection instead of the well ('OP1' 'STOP' 1 1 1 /) would
+    # leave the connection still shut. This behavior has been tested
+    # in the simulator.
+    pytest.param(
+        """
+    DATES
+     1 JAN 2000 /
+    /
+    COMPDAT
+     'OP1' 1 1 1 1 'OPEN' /
+    /
+    WELOPEN
+     'OP1' 'SHUT' /
+    /
+    DATES
+     1 FEB 2000 /
+    /
+    WELOPEN
+     'OP1' 'STOP' /
+    /
+    """,
+        pd.DataFrame(
+            columns=["DATE", "WELL", "I", "J", "K1", "K2", "OP/SH"],
+            data=[
+                [datetime.date(2000, 1, 1), "OP1", 1, 1, 1, 1, "SHUT"],
+                [datetime.date(2000, 2, 1), "OP1", 1, 1, 1, 1, "OPEN"],
+            ],
+        ),
+        id="welopen-shut-then-stop-on-well",
+    ),
+    pytest.param(
+        """
+    DATES
+     1 JAN 2000 /
+    /
+    COMPDAT
+     'OP1' 1 1 1 1 'OPEN' /
+    /
+    WELOPEN
+     'OP1' 'SHUT' 1 1 1 /
     /
     """,
         pd.DataFrame(
@@ -107,11 +191,84 @@ WELOPEN_CASES = [
                 [datetime.date(2000, 1, 1), "OP1", 1, 1, 1, 1, "SHUT"],
             ],
         ),
-        id="both_connection_and_completion_defined",
+        id="welopen-shut-on-connection",
+    ),
+    # Fail with ValueError when both I,J,K (3-5) and completions number (6-7)
+    pytest.param(
+        """
+    DATES
+     1 JAN 2000 /
+    /
+    COMPDAT
+     'OP1' 1 1 1 1 'OPEN' /
+    /
+    WELOPEN
+     'OP1' 'STOP' 1 1 1 /
+    /
+    """,
+        pd.DataFrame(
+            columns=["DATE", "WELL", "I", "J", "K1", "K2", "OP/SH"],
+            data=[
+                [datetime.date(2000, 1, 1), "OP1", 1, 1, 1, 1, "SHUT"],
+            ],
+        ),
+        id="welopen-stop-on-connection-is-shut",
+    ),
+    pytest.param(
+        """
+    DATES
+     1 JAN 2000 /
+    /
+    COMPDAT
+     'OP1' 1 1 1 1 'OPEN' /
+    /
+    WELOPEN
+     'OP1' 'POPN' 1 1 1 /
+    /
+    """,
+        pd.DataFrame(
+            columns=["DATE", "WELL", "I", "J", "K1", "K2", "OP/SH"],
+            data=[
+                [datetime.date(2000, 1, 1), "OP1", 1, 1, 1, 1, "OPEN"],
+            ],
+        ),
+        id="welopen-popn-on-connection-is-open",
+    ),
+    pytest.param(
+        """
+    DATES
+     1 JAN 2000 /
+    /
+    COMPDAT
+     'OP1' 1 1 1 1 'OPEN' /
+    /
+    WELOPEN
+     -- This points to a lumped connection, but COMPLUMP is missing
+     'OP1' 'SHUT' 1 1 1 1 1 /
+    /
+    """,
+        None,
+        id="complump_missing",
+        marks=pytest.mark.xfail(raises=ValueError),
+    ),
+    pytest.param(
+        """
+    DATES
+     1 JAN 2000 /
+    /
+    COMPDAT
+     'OP1' 1 1 1 1 'OPEN' /
+    /
+    WELOPEN
+     'OP2' 'SHUT' 1 1 1 /
+    /
+    """,
+        None,
+        id="operating-on-unknown-well",
         marks=pytest.mark.xfail(raises=ValueError),
     ),
     # Test J slicing
-    (
+    pytest.param(
         """
     DATES
      1 JAN 2000 /
@@ -131,10 +288,10 @@ WELOPEN_CASES = [
                 [datetime.date(2000, 1, 1), "OP1", 1, 1, 3, 3, "OPEN"],
             ],
         ),
+        id="j-slicing",
     ),
     # Test multiple connections to the same cell
-    # (ecl2df <= 0.13.1 would remove OP1 from this dataframe)
-    (
+    pytest.param(
         """
     DATES
      1 JAN 2000 /
@@ -144,8 +301,8 @@ WELOPEN_CASES = [
      'OP2' 1 1 1 1 'OPEN' /
     /
     WELOPEN
-     'OP1' 'SHUT'  /
-     'OP2' 'OPEN'  /
+     'OP1' 'SHUT' 0 0 0 /
+     'OP2' 'OPEN' 0 0 0 /
     /
     """,
         pd.DataFrame(
@@ -155,9 +312,10 @@ WELOPEN_CASES = [
                 [datetime.date(2000, 1, 1), "OP2", 1, 1, 1, 1, "OPEN"],
             ],
         ),
+        id="multiple-connnections-same-cell",
     ),
     # Test multiple time steps
-    (
+    pytest.param(
         """
     DATES
      1 MAY 2001 /
@@ -168,7 +326,7 @@ WELOPEN_CASES = [
     /
 
     WELOPEN
-     'OP1' 'SHUT' /
+     'OP1' 'SHUT' 0 0 0 /
     /
 
     TSTEP
@@ -200,8 +358,9 @@ WELOPEN_CASES = [
                 },
             }
         ),
+        id="multiple-time-steps",
     ),
-    (
+    pytest.param(
         """
     DATES
      1 MAY 2001 /
@@ -212,7 +371,7 @@ WELOPEN_CASES = [
     /
 
     WELOPEN
-     'OP1' 'OPEN' /
+     'OP1' 'OPEN' 0 0 0/
     /
 
     TSTEP
@@ -244,8 +403,9 @@ WELOPEN_CASES = [
                 },
             }
         ),
+        id="more-time-steps",
     ),
-    (
+    pytest.param(
         """
     DATES
      1 MAY 2001 /
@@ -257,7 +417,7 @@ WELOPEN_CASES = [
     /
 
     WELOPEN
-     'OP2' 'OPEN' /
+     'OP2' 'OPEN' 0 0 0/
     /
 
     DATES
@@ -268,7 +428,7 @@ WELOPEN_CASES = [
      'OP1' 34 111 32 32 'OPEN' /
     /
     WELOPEN
-     'OP1' 'SHUT' /
+     'OP1' 'SHUT' 0 0 0 /
     /
 
     DATES
@@ -276,8 +436,8 @@ WELOPEN_CASES = [
     /
 
     WELOPEN
-     'OP1' 'OPEN' /
-     'OP2' 'SHUT' /
+     'OP1' 'OPEN' 0 0 0 /
+     'OP2' 'SHUT' 0 0 0 /
     /
     """,
         pd.DataFrame(
@@ -315,8 +475,9 @@ WELOPEN_CASES = [
                 },
             }
         ),
+        id="date-stepping",
     ),
-    (
+    pytest.param(
         """
     DATES
      1 MAY 2001 /
@@ -343,7 +504,7 @@ WELOPEN_CASES = [
     /
 
     WELOPEN
-     'OP1' 'OPEN' /
+     'OP1' 'OPEN' 0 0 0 /
     /
     """,
         pd.DataFrame(
@@ -363,34 +524,9 @@ WELOPEN_CASES = [
                 },
             }
         ),
+        id="more-date-stepping",
     ),
-    (
-        """
-    DATES
-     1 MAY 2001 /
-    /
-
-    COMPDAT
-     'OP1' 1 1 1 1 'OPEN'  /
-    /
-
-    WELOPEN
-     'OP1' 'SHUT' /
-    /
-    """,
-        pd.DataFrame(
-            {
-                "WELL": {0: "OP1"},
-                "I": {0: 1},
-                "J": {0: 1},
-                "K1": {0: 1},
-                "K2": {0: 1},
-                "OP/SH": {0: "SHUT"},
-                "DATE": {0: datetime.date(2001, 5, 1)},
-            }
-        ),
-    ),
-    (
+    pytest.param(
         """
     DATES
      1 MAY 2001 /
@@ -423,34 +559,9 @@ WELOPEN_CASES = [
                 "DATE": {0: datetime.date(2001, 5, 1), 1: datetime.date(2001, 5, 2)},
             }
         ),
+        id="test-xx1",
     ),
-    (
-        """
-    DATES
-     1 MAY 2001 /
-    /
-
-    COMPDAT
-     'OP1' 1 1 1 2 'OPEN'  /
-    /
-
-    WELOPEN
-     'OP1' 'SHUT' 1 1 1 /
-    /
-    """,
-        pd.DataFrame(
-            {
-                "WELL": {0: "OP1", 1: "OP1"},
-                "I": {0: 1, 1: 1},
-                "J": {0: 1, 1: 1},
-                "K1": {0: 2, 1: 1},
-                "K2": {0: 2, 1: 1},
-                "OP/SH": {0: "OPEN", 1: "SHUT"},
-                "DATE": {0: datetime.date(2001, 5, 1), 1: datetime.date(2001, 5, 1)},
-            }
-        ),
-    ),
-    (
+    pytest.param(
         """
     DATES
      1 MAY 2001 /
@@ -461,38 +572,8 @@ WELOPEN_CASES = [
     /
 
     WELOPEN
-     'OP1' 'SHUT' /
-    /
-
-    WELOPEN
-     'OP1' 'OPEN' /
-    /
-    """,
-        pd.DataFrame(
-            {
-                "WELL": {0: "OP1"},
-                "I": {0: 1},
-                "J": {0: 1},
-                "K1": {0: 1},
-                "K2": {0: 1},
-                "OP/SH": {0: "OPEN"},
-                "DATE": {0: datetime.date(2001, 5, 1)},
-            }
-        ),
-    ),
-    (
-        """
-    DATES
-     1 MAY 2001 /
-    /
-
-    COMPDAT
-     'OP1' 1 1 1 1 'OPEN'  /
-    /
-
-    WELOPEN
-     'OP1' 'OPEN' /
-     'OP1' 'SHUT' /
+     'OP1' 'OPEN' 0 0 0 /
+     'OP1' 'SHUT' 0 0 0 /
     /
     """,
         pd.DataFrame(
@@ -506,8 +587,9 @@ WELOPEN_CASES = [
                 "DATE": {0: datetime.date(2001, 5, 1)},
             }
         ),
+        id="self-overwriting-records",
     ),
-    (
+    pytest.param(
         """
     DATES
      1 MAY 2001 /
@@ -518,7 +600,7 @@ WELOPEN_CASES = [
     /
 
     WELOPEN
-     'OP1' 'OPEN' /
+     'OP1' 'OPEN' 0 0 0 /
      'OP1' 'SHUT' 1 1 1 /
     /
     """,
@@ -533,38 +615,12 @@ WELOPEN_CASES = [
                 "DATE": {0: datetime.date(2001, 5, 1), 1: datetime.date(2001, 5, 1)},
             }
         ),
-    ),
-    # Test 0 values for ijk-connections
-    (
-        """
-    DATES
-     1 MAY 2001 /
-    /
-
-    COMPDAT
-     'OP1' 1 1 1 2 'OPEN'  /
-    /
-
-    WELOPEN
-     'OP1' 'SHUT' 0 0 0 2* /
-    /
-    """,
-        pd.DataFrame(
-            {
-                "WELL": {0: "OP1", 1: "OP1"},
-                "I": {0: 1, 1: 1},
-                "J": {0: 1, 1: 1},
-                "K1": {0: 2, 1: 1},
-                "K2": {0: 2, 1: 1},
-                "OP/SH": {0: "SHUT", 1: "SHUT"},
-                "DATE": {0: datetime.date(2001, 5, 1), 1: datetime.date(2001, 5, 1)},
-            }
-        ),
+        id="open-and-shut-slice-multiple-welopen",
     ),
     # Referencing multiple wells with wildcards
     # Wildcard structures are tested in test_common and need
     # not be tested here.
-    (
+    pytest.param(
         """
     DATES
       1 JAN 2000 /
@@ -575,7 +631,7 @@ WELOPEN_CASES = [
      'WI1' 3 3 3 3 'OPEN' /
     /
     WELOPEN
-     'B*H' 'SHUT' /
+     'B*H' 'SHUT' 0 0 0 /
     /
     """,
         pd.DataFrame(
@@ -586,10 +642,11 @@ WELOPEN_CASES = [
                 [datetime.date(2000, 1, 1), "WI1", 3, 3, 3, 3, "OPEN"],
             ],
         ),
+        id="multiple-wells-via-wildcard",
     ),
     # Test wildcard in wellname. A well that also matches the well template
     # but is defined later, is not SHUT
-    (
+    pytest.param(
         """
     DATES
       1 JAN 2000 /
@@ -600,7 +657,7 @@ WELOPEN_CASES = [
      'WI1' 3 3 3 3 'OPEN' /
     /
     WELOPEN
-     'OP*' 'SHUT' /
+     'OP*' 'SHUT' 0 0 0 /
     /
     DATES
       1 FEB 2000 /
@@ -618,11 +675,12 @@ WELOPEN_CASES = [
                 [datetime.date(2000, 2, 1), "OP3", 4, 4, 4, 4, "OPEN"],
             ],
         ),
+        id="wildcards-do-not-apply-to-future-wells",
     ),
 ]
 
 
-@pytest.mark.parametrize("test_input,expected", WELOPEN_CASES)
+@pytest.mark.parametrize("test_input, expected", WELOPEN_CASES)
 def test_welopen(test_input, expected):
     """Test with WELOPEN present"""
     deck = EclFiles.str2deck(test_input)
@@ -657,7 +715,7 @@ def test_welopen(test_input, expected):
       '*OP' NEW OP1 /
     /
     WELOPEN
-      '*OP' 'SHUT' /
+      '*OP' 'SHUT' 0 0 0 /
     /
     """,
             pd.DataFrame(
@@ -680,7 +738,7 @@ def test_welopen(test_input, expected):
       '*OP' NEW OP2 /
     /
     WELOPEN
-      '*OP' 'SHUT' /
+      '*OP' 'SHUT' 0 0 0 /
     /
     """,
             None,
@@ -703,7 +761,7 @@ def test_welopen(test_input, expected):
     /
     WELOPEN
       -- Shut the wells immediately, overriding OPEN in COMPDAT
-      '*OP' 'SHUT' /
+      '*OP' 'SHUT' 0 0 0 /
     /
     """,
             pd.DataFrame(
@@ -728,7 +786,7 @@ def test_welopen(test_input, expected):
     /
     WELOPEN
       -- In ecl2df, the WELOPEN is allowed to be before WLIST
-      '*OP' 'SHUT' /
+      '*OP' 'SHUT' 0 0 0 /
     /
     WLIST
       '*OP' NEW OP1 OP2 /
@@ -738,7 +796,7 @@ def test_welopen(test_input, expected):
       2 JAN 2000 /
     /
     WELOPEN
-      '*IN' 'SHUT' /
+      '*IN' 'SHUT' 0 0 0 /
     /
     """,
             pd.DataFrame(
@@ -763,7 +821,7 @@ def test_welopen(test_input, expected):
       'OP1' 1 1 1 1 'OPEN' /
     /
     WELOPEN
-      '*OP' 'SHUT' /
+      '*OP' 'SHUT' 0 0 0 /
     /
     DATES
       2 JAN 2000/
@@ -791,7 +849,7 @@ def test_welopen(test_input, expected):
       '*OP' NEW OP1 /
     /
     WELOPEN
-      '*OPS' 'SHUT' /
+      '*OPS' 'SHUT' 0 0 0 /
     /
     """,
             None,
@@ -818,7 +876,7 @@ def test_welopen(test_input, expected):
       '*OP' NEW OP1 /
     /
     WELOPEN
-      '*OP' 'SHUT' /
+      '*OP' 'SHUT' 0 0 0 /
     /
     """,
             pd.DataFrame(
@@ -837,6 +895,55 @@ def test_welopen_wlist(test_input, expected):
     pd.testing.assert_frame_equal(dfs["COMPDAT"][expected.columns], expected)
 
 
+def test_welopen_df():
+    """Test that we can obtain WELOPEN information when it applies on well state,
+    not on connections."""
+    deck = EclFiles.str2deck(
+        """
+    DATES
+     1 JAN 2000 /
+    /
+    COMPDAT
+     'OP1' 1 1 1 1 'OPEN' /
+    /
+    WELOPEN
+     'OP1' 'SHUT' / -- This is ignored for connections
+    /
+    """
+    )
+    print(compdat.deck2dfs(deck)["WELOPEN"])
+    pd.testing.assert_frame_equal(
+        compdat.deck2dfs(deck)["WELOPEN"],
+        pd.DataFrame(
+            columns=[
+                "DATE",
+                "WELL",
+                "I",
+                "J",
+                "K",
+                "C1",
+                "C2",
+                "STATUS",
+                "KEYWORD_IDX",
+            ],
+            data=[
+                [
+                    datetime.date(2000, 1, 1),
+                    "OP1",
+                    None,
+                    None,
+                    None,
+                    None,
+                    None,
+                    "SHUT",
+                    2,
+                ],
+            ],
+        ),
+        check_like=True,
+    )
+
+
 @pytest.mark.parametrize(
     "test_input, expected",
     [
@@ -1077,9 +1184,256 @@ WELOPEN
                 match="Defaults (zero) for C1/C2 is not implemented",
             ),
         ),
+        pytest.param(
+            """
+    DATES
+     1 JAN 2000 /
+    /
+    COMPDAT
+     'OP1' 1 1 1 1 'OPEN' /
+    /
+    COMPLUMP
+     'OP1' 1 1 1 1 1 /
+    /
+    WELOPEN
+     'OP1' 'SHUT' 1 1 1 1 1 /
+    /
+    """,
+            pd.DataFrame(
+                columns=["DATE", "WELL", "I", "J", "K1", "K2", "OP/SH"],
+                data=[
+                    [datetime.date(2000, 1, 1), "OP1", 1, 1, 1, 1, "SHUT"],
+                ],
+            ),
+            id="indices_and_complump_combined",
+        ),
+        pytest.param(
+            """
+    DATES
+     1 JAN 2000 /
+    /
+    COMPDAT
+     'OP1' 1 1 1 1 'OPEN' /
+     'OP1' 1 1 2 2 'OPEN' /
+    /
+    COMPLUMP
+     -- Assign completion number 1 and 2 to the two connections
+     'OP1' 1 1 1 1 1 /
+     'OP1' 1 1 2 2 2 /
+    /
+    WELOPEN
+     -- This is ok, gives shut well:
+     'OP1' 'SHUT' 1 1 1 1 1 /  -- must match both i,j,k and complump
+     -- The following is ignored, because completion 2 is not at 1,1,1
+     'OP1' 'SHUT' 1 1 1 2 2 /
+    /
+    """,
+            pd.DataFrame(
+                columns=["DATE", "WELL", "I", "J", "K1", "K2", "OP/SH"],
+                data=[
+                    [datetime.date(2000, 1, 1), "OP1", 1, 1, 2, 2, "OPEN"],
+                    [datetime.date(2000, 1, 1), "OP1", 1, 1, 1, 1, "SHUT"],
+                ],
+            ),
+            id="indices_and_complump_combined_2",
+        ),
     ],
 )
 def test_welopen_complump(test_input, expected):
+    """Test the welopen_complump functionality through Eclipse decks"""
     deck = EclFiles.str2deck(test_input)
     dfs = compdat.deck2dfs(deck)
     pd.testing.assert_frame_equal(dfs["COMPDAT"][expected.columns], expected)
+
+
+@pytest.mark.parametrize(
+    "welopen, complump, expected",
+    [
+        pytest.param([], [], [], id="empty_input_empty_output"),
+        pytest.param(
+            [{"FOO": "BAR"}],
+            [],
+            [{"FOO": "BAR"}],
+            id="empty_complump_pass_through_welopen",
+        ),
+        pytest.param(
+            [{"WELL": "OP1", "C1": None, "C2": None}],
+            [],
+            [{"WELL": "OP1", "C1": None, "C2": None}],
+            id="pass_through_None_complumps",
+        ),
+        pytest.param(
+            [{"WELL": "OP1", "DATE": datetime.date(2000, 1, 1), "C1": 1, "C2": 1}],
+            [
+                {
+                    "WELL": "OP1",
+                    "DATE": datetime.date(2000, 1, 1),
+                    "STATUS": "OPEN",
+                    "I": 2,
+                    "J": 2,
+                    "K1": 2,
+                    "K2": 2,
+                    "N": 1,
+                }
+            ],
+            [
+                {
+                    "WELL": "OP1",
+                    "DATE": datetime.date(2000, 1, 1),
+                    "C1": None,
+                    "C2": None,
+                    "I": 2,
+                    "J": 2,
+                    "K": 2,
+                }
+            ],
+            id="basic",
+        ),
+        pytest.param(
+            [{"WELL": "OP1", "DATE": datetime.date(2000, 1, 1), "C1": 1, "C2": 1}],
+            [
+                {
+                    "WELL": "OP1",
+                    "DATE": datetime.date(2000, 1, 1),
+                    "STATUS": "OPEN",
+                    "I": 2,
+                    "J": 2,
+                    "K1": 2,
+                    "K2": 3,
+                    "N": 1,
+                }
+            ],
+            [],
+            id="k2_not_equal_kq_in_complump",
+            marks=pytest.mark.xfail(raises=ValueError),
+        ),
+    ],
+)
+def test_welopen_complump_direct(welopen, complump, expected):
+    """Test the welopen_complump directly.
+
+    For situations not that easy to reach via string input."""
+    pd.testing.assert_frame_equal(
+        compdat.expand_complump_in_welopen_df(
+            pd.DataFrame(welopen), pd.DataFrame(complump)
+        ),
+        pd.DataFrame(expected),
+        check_dtype=False,
+    )
+
+
+@pytest.mark.parametrize(
+    "compdat_rows, welopen_rows, wlist_rows, complump_rows, expected_rows",
+    [
+        pytest.param([], [], [], [], [], id="emptyinput"),
+        pytest.param(
+            [{"WELL": "OP1"}],
+            [],
+            [{"ACTION": "ADD"}],
+            [],
+            [],
+            id="non-expanded_wlist",
+            # "The WLIST dataframe must be expanded through expand_wlist()"
+            marks=pytest.mark.xfail(raises=ValueError),
+        ),
+        pytest.param(
+            [{"WELL": "OP1", "I": 1, "J": 1, "K1": 1, "K2": 1}],
+            [{"WELL": "OP1", "STATUS": "SHUT", "I": 0, "J": 0, "K": 0}],
+            [],
+            [],
+            [],
+            id="no_keyword_idx",
+            # Column KEYWORD_IDX must be present in compdat rows
+            marks=pytest.mark.xfail(raises=KeyError),
+        ),
+        pytest.param(
+            [
+                {
+                    "WELL": "OP1",
+                    "I": 1,
+                    "J": 1,
+                    "K1": 1,
+                    "K2": 1,
+                    "KEYWORD_IDX": 1,
+                    "DATE": None,
+                }
+            ],
+            [
+                {
+                    "WELL": "OP1",
+                    "STATUS": "SHUT",
+                    "I": 0,
+                    "J": 0,
+                    "K": 0,
+                    "KEYWORD_IDX": 2,
+                    "DATE": None,
+                }
+            ],
+            [],
+            [],
+            [
+                {
+                    "WELL": "OP1",
+                    "I": 1,
+                    "J": 1,
+                    "K1": 1,
+                    "K2": 1,
+                    "KEYWORD_IDX": 2,
+                    "DATE": None,
+                    "OP/SH": "SHUT",
+                }
+            ],
+            id="working_example",
+        ),
+        pytest.param(
+            [
+                {
+                    "WELL": "OP1",
+                    "I": 1,
+                    "J": 1,
+                    "K1": 1,
+                    "K2": 1,
+                    "KEYWORD_IDX": 1,
+                    "DATE": None,
+                }
+            ],
+            [
+                {
+                    "WELL": "OP1",
+                    "STATUS": "SHUT",
+                    "I": 1,  # Either all zero or none nonzero
+                    "J": 0,
+                    "K": 0,
+                    "KEYWORD_IDX": 2,
+                    "DATE": None,
+                }
+            ],
+            [],
+            [],
+            [],
+            id="invalid welopen",
+            # " A WELOPEN keyword contains data that could not be parsed"
+            marks=pytest.mark.xfail(raises=ValueError),
+        ),
+    ],
+)
+def test_applywelopen(
+    compdat_rows, welopen_rows, wlist_rows, complump_rows, expected_rows
+):
+    print(
+        compdat.applywelopen(
+            pd.DataFrame(compdat_rows),
+            pd.DataFrame(welopen_rows),
+            pd.DataFrame(wlist_rows),
+            pd.DataFrame(complump_rows),
+        ).to_dict(orient="records")
+    )
+    pd.testing.assert_frame_equal(
+        compdat.applywelopen(
+            pd.DataFrame(compdat_rows),
+            pd.DataFrame(welopen_rows),
+            pd.DataFrame(wlist_rows),
+            pd.DataFrame(complump_rows),
+        ),
+        pd.DataFrame(expected_rows),
+    )
